#!/usr/bin/perl
# ナイーブベイズ推定
# -c 学習する（任意）
# -b カテゴリファイルを指定する（未指定時 category100.txt）
# -f 判別する（必須）

use strict;
use warnings;
use utf8;
use Algorithm::NaiveBayes;
use Algorithm::NaiveBayes::Model::Frequency;
use Storable;
use Getopt::Std;
use Data::Dumper;
use Encode qw/decode encode/;
use LazyFile;

getopt('bf'); #Create, Base File
use vars qw / $opt_c $opt_b $opt_f /;

if (!defined($opt_f)) { 
  die "usage: need -f filename\n";
}
$opt_b ||= "category100.txt";

my %list;
my $DB = 'bayes.db';
if ($opt_c) {
  unlink($DB);
}

my $bayes = &load_instance;

if ($opt_c) {
  &calc_bayes;
}

%list = &get_hash($opt_f);
my $base = $bayes->predict(
    attributes => {}
);
my $result = $bayes->predict(
    attributes => {%list}
);
foreach my $title (keys %$base) {
  $result->{$title} = $result->{$title} - $base->{$title};
  #事後確率の変動率 = 事後確率（情報あり） - 事後確率（情報なし）
}

#print Dumper($result);
my @sorted = sort { - ($result->{$a} <=> $result->{$b}) } keys %$result;
my $i = 0;
foreach my $title (@sorted) {
  next if $i++ >= 5; #最大五つまで表示
  next if $result->{$title} <= 0; # 0以下は非表示
  print encode("shiftjis", $title), " => ", $result->{$title}, "\n";
  
}

sub load_instance {
  eval {$bayes = Storable::retrieve($DB)} if -e $DB;
  $bayes ||= Algorithm::NaiveBayes->new(purge => 0);
  return $bayes;
}

sub calc_bayes {
  my @category_list = readLines($opt_b, "shiftjis");

  foreach my $category (@category_list) {
    my $sjis_category = encode("shiftjis", $category);
    next unless -f "index/$sjis_category.txt";
    
    print $sjis_category, "\n";
    my @sjis_title_list = readLines("index/$sjis_category.txt", "shiftjis");

    foreach my $title (@sjis_title_list) {
      #next if $title =~ m/^\d+年$/;
      next if $title =~ m/^\d+月\d+日$/;
      
      my $sjis_title = encode("shiftjis", $title);
      my $path = "wiki_m/$sjis_title.txt";
      if (-f $path) {
        print $sjis_category, " - ", $sjis_title, "\n";
        %list = &get_hash($path);
        $bayes->add_instance(
            attributes => {%list},
            label => $category,
        );
      }
    }
  }

  $bayes->train;
  Storable::store($bayes => $DB);
}

sub get_hash {
  my $file = $_[0];
  my %hash;
  open INFILE, "<", $file or die "Cannot open file: $file";
  while (<INFILE>) {
    #chomp();
    s/\r|\n//g;
    s/^ +//;
    s/ +$//;
    my @words = split(/ +/);
    foreach (@words) {
      #next if (m/、|。/);
      if (!$hash{$_}) {$hash{$_} = 0};
      $hash{$_} = $hash{$_}+1;
    }
  }
  return %hash;
}
