#!/usr/bin/perl
# Wikipediaからの辞書片の作成

use strict;
use warnings;
use Data::Dumper;
use Encode qw/decode encode/;
use utf8;
use LazyFile;

my $filename = shift @ARGV || 'head10000.txt';

open(WIKI, "<:encoding(utf8)", $filename);

my %doc = ();
my $utf8_title = '';
my $title;

# 公式タイトルの読み込み
my @list_title = readLines("title.txt", "utf8");
my $len_title = @list_title;
my %hash_title = ();
foreach my $title (@list_title) {
	#print encode("shiftjis", $title), "\n";
	$hash_title{$title} = 1;
}
print "title ok\n";

my $is_title = 0;
my $is_in_text = 0;
my $i = 0;
while (my $line = <WIKI>) {
	my $utf8 = $line;
	$utf8 =~ s/\n/ /g;
	if ($utf8 =~ m/<title/) {
		if ($utf8 =~ m!<title[^>]*>(?:[A-Za-z0-9]+: ?)?(.*?)</title>!) {
			$utf8_title = $1;
			$is_title = $hash_title{$utf8_title};
			print $i++, '/', $len_title, "\n";
		}
	}
	next unless $is_title;
	if ($utf8 =~ m!<text[^>]*>(.*?)</text>!) {
		my $text = $1;
		$is_in_text = 0;
		$doc{$utf8_title} .= $text if $is_title;
	} else {
		if ($utf8 =~ m!<text[^>]*>(.*)!) {
			my $text = $1;
			$is_in_text = 1 if $is_title;
			$doc{$utf8_title} .= $text . "\n" if $is_title;
		} elsif ($utf8 =~ m!(.*?)</text>!) {
			my $text = $1;
			$is_in_text = 0;
			$doc{$utf8_title} .= $text if $is_title;
		} elsif ($utf8 =~ m!^(.*)! && $is_in_text) {
			my $text = $1;
			$doc{$utf8_title} .= $text . "\n" if $is_title;
		}
	}
}

mkdir('wiki');
chdir('wiki');
writeFile('title.txt', '') unless (-f 'title.txt');

$i = 0;
foreach my $key (keys %doc) {
	my $value = $doc{$key};
	$value =~ s!==?=?=? ?([^= ]*) ?==?=?=?!$1!g;
	$value =~ s!\*+ ?(.*?)!$1!g;
	$value =~ s!'''?([^']*?)'''?!$1!g; #'
	$value =~ s!\[\[(?:[^\]|:]*)(?::[^\]|]*)+\]\]!!g;
	#$value =~ s!\[\[([^\]|]*)(?:\|[^\]]*)?\]\]!$1!g;
	$value =~ s!{{([^}]*)(?:\|[^}]*)*}}!!g;
	$value =~ s!(?:\[\[)?(ファイル|File|画像|イメージ|Image):[^\r\n]*!!ig;
	$value =~ s!(?:\[\[)?(カテゴリ|category):[^\r\n]*!!ig;
	$value =~ s!&lt;!<!g;
	$value =~ s!&gt;!>!g;
	$value =~ s!&quot;!"!g; #"
	$value =~ s!&amp;!&!g;
	$value =~ s!<[^>]+?>!!g;
	$value =~ s!</[^>]+?>!!g;
	
	print $i++, ":", encode("shiftjis", $key), "\n";
	my $utf8_url = url_encode(encode("utf8", $key));
	my $utf8_category = substr($utf8_url, 0, 6);
	eval {
		unless (-f "$utf8_category/$utf8_url") {
			mkdir($utf8_category);
			writeFile("$utf8_category/$utf8_url", encode("shiftjis", $value));
			appendFile('title.txt', encode("shiftjis", $key));
		}
	}
}
close(WIKI);

sub url_encode {
  my $str = shift;
  $str =~ s/([^\w ])/'%'.unpack('H2', $1)/eg;
  $str =~ tr/ /+/;
  return $str;
}

sub url_decode {
  my $str = shift;
  $str =~ tr/+/ /;
  $str =~ s/%([0-9A-Fa-f][0-9A-Fa-f])/pack('H2', $1)/eg;
  return $str;
}