#!/usr/bin/perl
# MeCabを使った分かち書き
# カテゴリを引数に取る

use strict;
use warnings;
use Data::Dumper;
use Encode qw/decode encode/;
use utf8;
use LazyFile;

#必要なディレクトリの作成
mkdir('index');
mkdir('wiki_t');
mkdir('wiki_m');

# カテゴリの読み込み
my @category_list = readLines(shift @ARGV || "category100.txt", "shiftjis");

foreach my $category (@category_list) {
  my $sjis_category = encode("shiftjis", $category);
  my $title = $sjis_category;
  
  print encode("shiftjis", "■ "), $title, "\n";
  my $sjis_title = encode("shiftjis", $title);

  my @wiki_links = &wiki_links($category); #2回までリンクを辿る
  #@wiki_links = &wiki_links(@wiki_links); #3回辿ってみる
  #@wiki_links = @wiki_links[0..10000]; #絞り込む
  my %wiki_links;
  foreach my $title (@wiki_links) {
    $wiki_links{$title} = 1;
  }
  @wiki_links = sort keys %wiki_links;
  writeFile("index/$sjis_category.txt", '');
  foreach my $title (@wiki_links) {
    #next if $title =~ m/^\d+年$/;
    next if $title =~ m/^\d+月\d+日$/;

    my $path = '';
    eval { $path = wiki_url(decode("shiftjis", $title)); };
    next unless $path;
    
    next unless -f $path;
    print $sjis_category, " - ", $title, "\n";
    appendFile("index/$sjis_category.txt", $title);
    
    #next;

    my $sjis_title = $title;
    #next if -f "wiki_t/$sjis_title.txt";
    eval {
      my $wiki = readText($path);
      my $utf8_wiki = decode("shiftjis", $wiki);
      #next if $utf8_wiki =~ m/#転送|#REDIRECT/i;
      
      $wiki =~ s/\[\[([^\]|]*)(\|[^\]]*)?\]\]/$1/g;
      my $wiki_t_path = "wiki_t/$sjis_title.txt";
      unless (-f $wiki_t_path) {
        writeFile($wiki_t_path, $wiki);
        $_ = `mecab -O wakati $wiki_t_path`;
        writeFile("wiki_m/$sjis_title.txt", $_) if $_;
      }
    }
  }
}

sub wiki_links {
  my @result = (@_);
  my (@title_list) = @_;
  foreach my $title (@title_list) {
    my $wiki_url = '';
    $wiki_url = wiki_url($title);
    eval { $wiki_url = wiki_url(decode("shiftjis", $title)); };
    next unless -f $wiki_url;
    my $wiki = readText($wiki_url);
    my @links = $wiki =~ m/\[\[([^\]|]*)(?:\|[^\]]*)?\]\]/g;
    foreach my $link (@links) {
      #print $link, "\n";
      push @result, $link;
    }
  }
  return @result;
}

sub wiki_url {
  my $title = shift;
  my $utf8_url = url_encode(encode("utf8", $title));
  my $utf8_category = substr($utf8_url, 0, 6);
  my $path = "wiki/$utf8_category/$utf8_url";
  return $path;
}

sub url_encode {
  my $str = shift;
  $str =~ s/([^\w ])/'%'.unpack('H2', $1)/eg;
  $str =~ tr/ /+/;
  return $str;
}

sub url_decode {
  my $str = shift;
  $str =~ tr/+/ /;
  $str =~ s/%([0-9A-Fa-f][0-9A-Fa-f])/pack('H2', $1)/eg;
  return $str;
}
